//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***

#include <hpgcc3.h>
extern "C" {
#include <ggl.h>
}

#include "memblock.h"
#include "gstring.h"
#include "gui.h"
#include "resources.h"




// CLASS gSoftMenu

gSoftMenu::gSoftMenu(gControl& _Parent,int _Style)
: gControl()
{
	Parent=&_Parent;	// DO NOT REGISTER AS A CHILD OF THE FORM
	if(!_Style) _Style=S_SINGLESOFTMENU;
	Style=_Style;
	Visible=TRUE;
	Numitems=0;
	Items=NULL;
	ShowItem=0;
	((gForm *)Parent)->AddSoftMenu(this); // REGISTER AS THE FORM'S SOFTMENU;
}

gSoftMenu::gSoftMenu(gControl& _Parent,int _Style,struct gSoftMenuItem *List, int numitems)
: gControl()
{
	Parent=&_Parent;	// DO NOT REGISTER AS A CHILD OF THE FORM
	((gForm *)Parent)->SoftMenu=this;	// REGISTER AS THE SOFTMENU OF THE FORM
	if(!_Style) _Style=S_SINGLESOFTMENU;
	Style=_Style;
	Visible=TRUE;
	Numitems=0;
	Items=NULL;
	ShowItem=0;
	AddItemList(List,numitems);
}

gSoftMenu::~gSoftMenu()
{
	if(Items!=NULL) delete Items;
}

void gSoftMenu::Invalidate(gControl *Sender)
{
	int rows;
	if(Style&S_EXSOFTMENU) { rows=3*SOFTMENUHEIGHT; }
	else { rows=SOFTMENUHEIGHT; }
	gApplication *ap=GetApplication();
	gUpdate u;
	u.clipx=0;
	u.clipy=ap->sizey-rows-1;
	u.clipx2=ap->sizex-1;
	u.clipy2=ap->sizey-1;
	ap->Invalidate(&u);
}

void gSoftMenu::SetVisible(BOOL _Visible)
{
	if(Visible!=_Visible) Invalidate();
	Visible=_Visible;
}
BOOL gSoftMenu::IsVisible()
{
	return Visible;
}

void gSoftMenu::ClearItems()
{
if(Items!=NULL) {
	delete Items;
	Items=NULL;
	Numitems=0;
}
}


void gSoftMenu::AddItem(const char *Name,gEvHandler Function)
{
	gSoftMenuItem *newlist=new gSoftMenuItem[Numitems+1];
	
	if(!newlist) return;
	if(Items) {
	memcpy(newlist,Items,Numitems*sizeof(gSoftMenuItem));
	delete Items;
	}
	Items=newlist;
	strncpy(Items[Numitems].Name,Name,7);
	Items[Numitems].Name[7]=0;
	Items[Numitems].Action=Function;
	Items[Numitems].State=0;
	Numitems++;
	Invalidate();

}



void gSoftMenu::AddItemList(struct gSoftMenuItem *List,int nitems)
{
	gSoftMenuItem *newlist=new gSoftMenuItem[Numitems+nitems];
	
	if(!newlist) return;
	if(Items) {
	memcpy(newlist,Items,Numitems*sizeof(gSoftMenuItem));
	delete Items;
	}
	Items=newlist;
	memcpy(Items+Numitems,List,nitems*sizeof(gSoftMenuItem));
	Numitems+=nitems;	
	Invalidate();

}

void gSoftMenu::ChangeItem(int number,const char *Name,gEvHandler Function)
{
	if(number>=Numitems) return;
	strncpy(Items[number].Name,Name,7);
	Items[number].Name[7]=0;
	Items[number].Action=Function;
	Items[number].State=0;
	Invalidate();
	
}


void gSoftMenu::DrawMenuLabel(gControl *P,int x,int y,int item)
{
	// TODO: Check State and show proper icon
	
  P->DrawIcon(x,y,iSoftMenu,22,6);
  
  if(item>=Numitems || item<-1 ) return;
  
  gFont f(5);
  
  if(item!=-1) {
  
  int width=f.TextWidth(Items[item].Name);
  
  if(width>20) {
	  // TRUNCATE STRING
	  int k=0;
	  while(Items[item].Name[k]!=0 ) ++k;
	  
	  while(k>0 && width>20) {
	  --k;
	  Items[item].Name[k]=0;	// REMOVE ONE CHARACTER
	  width=f.TextWidth(Items[item].Name);
	  }
  }
  // CENTER THE TEXT
  
  // TODO: CHECK STATE AND DISPLAY TEXT ACCORDINGLY
  
  P->DrawText(x+11-width/2,y+1,Items[item].Name,&f,GBLACK);
  
  }
  else {
	  
	  int width=f.TextWidth("Next");
	  P->DrawText(x+11-width/2,y+1,"Next",&f,GBLACK);
	  
  }

}



void gSoftMenu::Update(gUpdate *u)
{
	int n,nvis,x,y;
	
	// DRAW DIRECTLY TO SCREEN, NOT TO PARENT'S CLIENT AREA
	if(!Visible) return;
	
	gApplication *App=GetApplication();

	x=0;

	if(Style==S_EXSOFTMENU) y=App->sizey-SOFTMENUHEIGHT*3;
	else y=App->sizey-SOFTMENUHEIGHT;
	

		
	n=ShowItem;
	// DRAW TOP LINE
	for(nvis=0;nvis<6;nvis++,x+=22,n++) {
		DrawMenuLabel(App,x,y,n);
	}
	
	if(Style==S_EXSOFTMENU) {
	// DRAW 3 ROWS - EXTENDED MENU
	for(nvis=0,x=0,y+=SOFTMENUHEIGHT;nvis<3;nvis++,x+=22,n++) {
		DrawMenuLabel(App,x,y,n);
	}
	for(nvis=0,x=0,y+=SOFTMENUHEIGHT;nvis<2;nvis++,x+=22,n++) {
		DrawMenuLabel(App,x,y,n);
	}
	
	if(Numitems>12) 
		DrawMenuLabel(App,x,y,-1);
	else 
		DrawMenuLabel(App,x,y,n);

	}
		
	
}

BOOL gSoftMenu::DefaultEvent(gEvent *e)
{
	int item=0;
	if(e->Type==GEVENT_KEYB) {
		// IMPLEMENT BASIC KEYBOARD HANDLING
	switch(e->Arg)
	{
	case KM_PRESS | KB_A:
	case KM_PRESS | KB_A | SHIFT_LS:
	case KM_PRESS | KB_A | SHIFT_LS | SHIFT_LSHOLD:
	item=1;
	break;
	case KM_PRESS | KB_B:
	case KM_PRESS | KB_B | SHIFT_LS:
	case KM_PRESS | KB_B | SHIFT_LS | SHIFT_LSHOLD:
	item=2;
	break;
	case KM_PRESS | KB_C:
	case KM_PRESS | KB_C | SHIFT_LS:
	case KM_PRESS | KB_C | SHIFT_LS | SHIFT_LSHOLD:
	item=3;
	break;
	case KM_PRESS | KB_D:
	case KM_PRESS | KB_D | SHIFT_LS:
	case KM_PRESS | KB_D | SHIFT_LS | SHIFT_LSHOLD:
	item=4;
	break;
	case KM_PRESS | KB_E:
	case KM_PRESS | KB_E | SHIFT_LS:
	case KM_PRESS | KB_E | SHIFT_LS | SHIFT_LSHOLD:
	item=5;
	break;
	case KM_PRESS | KB_F:
	case KM_PRESS | KB_F | SHIFT_LS:
	case KM_PRESS | KB_F | SHIFT_LS | SHIFT_LSHOLD:
	item=6;
	break;
	case KM_PRESS | KB_G:
	case KM_PRESS | KB_G | SHIFT_LS:
	case KM_PRESS | KB_G | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) item=7;
	break;
	case KM_PRESS | KB_H:
	case KM_PRESS | KB_H | SHIFT_LS:
	case KM_PRESS | KB_H | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) item=8;
	break;
	case KM_PRESS | KB_I:
	case KM_PRESS | KB_I | SHIFT_LS:
	case KM_PRESS | KB_I | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) item=9;
	break;
	case KM_PRESS | KB_J:
	case KM_PRESS | KB_J | SHIFT_LS:
	case KM_PRESS | KB_J | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) item=10;
	break;
	case KM_PRESS | KB_K:
	case KM_PRESS | KB_K | SHIFT_LS:
	case KM_PRESS | KB_K | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) item=11;
	break;
	case KM_PRESS | KB_L:
	case KM_PRESS | KB_L | SHIFT_LS:
	case KM_PRESS | KB_L | SHIFT_LS | SHIFT_LSHOLD:
	if(Style&S_EXSOFTMENU) {
		if(Numitems>12) item=-1;
		else item=12;
	}
	else {
		if(Numitems>6) item=-1;
	}
	break;
	default: 
		break;

	}
	if(item<0) {
	NextPage();
	return TRUE;
	}
	if(item>0) {
	item+=ShowItem-1;
	if(item<0 || item>=Numitems) return FALSE;
	return Items[item].Action(e,this);
	}
	}
	
	
	return FALSE;
	
}


void gSoftMenu::NextPage()
{
	int itemsperpage;
	if(Style&S_EXSOFTMENU) { itemsperpage=11; }
	else { itemsperpage=5; }
	if(Numitems>itemsperpage+1) {
		ShowItem+=itemsperpage;
		if(ShowItem>=Numitems) ShowItem=0;
		
		Invalidate();
	}
	else ShowItem=0;
	
}

void gSoftMenu::PrevPage()
{
	int itemsperpage;
	if(Style&S_EXSOFTMENU) { itemsperpage=11; }
	else { itemsperpage=5; }
	if(Numitems>itemsperpage+1) {
		if(ShowItem<itemsperpage) {
			ShowItem=0;
			while(ShowItem<Numitems) ShowItem+=itemsperpage;
		}
		ShowItem-=itemsperpage;

		Invalidate();
	}
	else ShowItem=0;
	
}
