//& *** (c) 2006-2011 The HPGCC3 Team ***
//& Claudio Lapilli
//& Ingo Blank
//&
//& This file is licensed under the terms and conditions of the
//& HPGCC3 license that is included with the source distribution.
//& *** (c) 2006-2011 The HPGCC3 Team ***



// $Id: sat3_matrix.c 547 2008-02-06 01:01:02Z claudio $

#include <hpgcc3.h>
#include <master_globals.h>


SAT_OBJECT sat3_creatematrix(SAT_OBJECT where,int rows,int cols,SAT_OBJECT fillobj,int allocsize )
{
	if(!fillobj) return 0;
	int size=sat3_sizeof(fillobj);
	if(allocsize< (10+rows*(10+cols*size))) return 0;	// ALLOCSIZE IS INSUFFICIENT TO CONTAIN THE MATRIX
    where=sat3_alloc(where,10+allocsize);
	if(where==0) return 0;
	if(rows==0 || cols==0) { rows=rows+cols; cols=0; }	// CREATE 1D VECTOR


	// CREATE MATRIX FULL OF THE GIVEN FILL OBJECT
	
	sat3_poke(where,SAT_DOMATRIX,5);
	
	SAT_OBJECT ptr=where+5;
	int count;
	while(rows--) {
	if(cols) {				// CREATE 2D MATRIX
		sat3_poke(ptr,SAT_DOMATRIX,5);
		ptr+=5;
		count=cols;
		while(count--) {
			sat3_copyobject(ptr,fillobj);
			ptr+=size;
		}
		sat3_poke(ptr,SAT_SEMI,5);
		ptr+=5;
		
	}
	else {					// CREATE 1D VECTOR
		sat3_copyobject(ptr,fillobj);
		ptr+=size;
	}	
		
	}	
		
    sat3_poke(ptr,SAT_SEMI,5);

    return where;
}

// GET A ROW OF THE MATRIX, OR AN ELEMENT IN A 1D VECTOR

SAT_OBJECT sat3_matrixgetrow(SAT_OBJECT matrix,int row)
{
	if(!matrix) return 0;
if(sat3_peek(matrix,5)!=SAT_DOMATRIX)  return 0;
if(row<0) return 0;

matrix+=5;
while( (row--) && (sat3_peek(matrix,5)!=SAT_SEMI)) matrix=sat3_skipob(matrix);

if(sat3_peek(matrix,5)==SAT_SEMI) return 0; 	// row OUT OF MATRIX

return matrix;

}

// GET AN ELEMENT OF THE MATRIX

SAT_OBJECT sat3_matrixget(SAT_OBJECT matrix,int row, int col)
{
	SAT_OBJECT m=sat3_matrixgetrow(matrix,0);
	if(!m) return 0;
	if(sat3_peek(m,5)!=SAT_DOMATRIX) {
		// ONE-DIMENSIONAL MATRIX
		if(row*col) return 0;
		row=row+col;
		col=0;
	}


	matrix=sat3_matrixgetrow(matrix,row);

	if(!matrix) return 0;
	if(sat3_peek(matrix,5)!=SAT_DOMATRIX) {
		return matrix;
	}

	return sat3_matrixgetrow(matrix,col);

}

// IF OBJ POINTS TO A MATRIX, GET FIRST ROW,
// IF OBJ POINTS TO A ROW, GET FIRST ELEMENT
SAT_OBJECT sat3_matrixfirst(SAT_OBJECT obj)
{
	if(!obj) return 0;
if(sat3_peek(obj,5)!=SAT_DOMATRIX) return 0;
obj+=5;
return (sat3_peek(obj,5)!=SAT_SEMI)? obj:0;
}

// IF OBJ POINTS TO AN ELEMENT, GET NEXT OBJECT IN ROW ORDER,
// IF OBJ POINTS TO A ROW, GET NEXT ROW
SAT_OBJECT sat3_matrixnext(SAT_OBJECT obj)
{
return sat3_listnext(obj);
}


// GET MATRIX SIZE, RETURNS FALSE IF MATRIX SIZE IS INVALID
BOOL sat3_matrixsize(SAT_OBJECT matrix,int *rowptr,int *colptr)
{
if(!matrix) return 0;
if(sat3_peek(matrix,5)!=SAT_DOMATRIX) return 0;
	
SAT_OBJECT 	row;
int rows=0,cols=-1,rrow,rcol;
	
row=sat3_matrixfirst(matrix);
while(row) {
if(sat3_peek(row,5)==SAT_SEMI) break;
++rows;
if(!sat3_matrixsize(row,&rrow,&rcol)) {
if(cols<0) cols=0;
else if(cols!=0) return 0;	// COLUMNS HAVE DIFFERENT SIZE
}
else {
if(cols<0) cols=rrow;
else if(cols!=rrow) return 0;		// COLUMNS HAVE DIFFERENT SIZE
}
row=sat3_listnext(row);
}

if(!rows) return 0;

*rowptr=rows;
*colptr=cols;
return 1;

}


// ADD A ROW TO THE MATRIX
BOOL sat3_matrixrowinsert(SAT_OBJECT matrix,int rowpos,SAT_OBJECT fillobj)
{


// GET MATRIX SIZE
int rows,cols;
if(!sat3_matrixsize(matrix,&rows,&cols)) return 0;

// GET POSITION TO INSERT OBJECT
SAT_OBJECT position;
if(rowpos==rows) position=sat3_skipob(matrix)-5;
else position=sat3_matrixgetrow(matrix,rowpos);
if(!position) return 0;

SAT_OBJECT newrow;
if(cols) {
// 2-DIMENSIONAL MATRIX
// CREATE A ROW
newrow=sat3_creatematrix(ALLOC_HEAP,cols,0,fillobj, sat3_sizeof(fillobj)*cols+10);
}
else newrow=fillobj;

if(!newrow) return 0;
// INSERT ROW INTO MATRIX

int result=sat3_listinsert(position,newrow);

// FREE NEW ROW
sat3_free(newrow);

return result;	
}

// ADD A ROW TO THE MATRIX
BOOL sat3_matrixrowremove(SAT_OBJECT matrix,int rowpos)
{
// GET POSITION TO REMOVE OBJECT
SAT_OBJECT position=sat3_matrixgetrow(matrix,rowpos);
if(!position) return 0;

// REMOVE ROW INTO MATRIX

return sat3_listremove(position);
}


// INSERT A COLUMN IN THE MATRIX
BOOL sat3_matrixcolinsert(SAT_OBJECT matrix,int colpos,SAT_OBJECT fillobj)
{
// GET MATRIX SIZE
int rows,cols;
if(!sat3_matrixsize(matrix,&rows,&cols)) return 0;

if(!cols) return sat3_matrixrowinsert(matrix,colpos,fillobj);	// ONE-DIMENSIONAL

if(colpos<0 || colpos>cols) return 0;

// INSERT 1 ELEMENT ON EACH ROW

SAT_OBJECT row=sat3_matrixfirst(matrix),position;
while(row) {
if(colpos==cols) position=sat3_skipob(row)-5;
else position=sat3_matrixgetrow(row,colpos);
if(!position) return 0;
if(!sat3_listinsert(position,fillobj)) return 0;
row=sat3_matrixnext(row);
}

return 1;
	
}


// INSERT A COLUMN IN THE MATRIX
BOOL sat3_matrixcolremove(SAT_OBJECT matrix,int colpos)
{
// GET MATRIX SIZE
int rows,cols;
if(!sat3_matrixsize(matrix,&rows,&cols)) return 0;

if(!cols) return sat3_matrixrowremove(matrix,colpos);	// ONE-DIMENSIONAL

if(colpos<0 || colpos>=cols) return 0;

// REMOVE 1 ELEMENT ON EACH ROW

SAT_OBJECT row=sat3_matrixfirst(matrix),position;
while(row) {
position=sat3_matrixgetrow(row,colpos);
if(!position) return 0;
if(!sat3_listremove(position)) return 0;
row=sat3_matrixnext(row);
}

return 1;
	
}

// STORE AN OBJECT WITHIN THE MATRIX

BOOL sat3_matrixput(SAT_OBJECT matrix,int row,int col,SAT_OBJECT obj)
{
SAT_OBJECT position=sat3_matrixget(matrix,row,col);

if(!sat3_listremove(position)) return 0;

return sat3_listinsert(position,obj);

}

